
/********************************************************************
 *
 * FILE:
 *	echo.c
 *
 * DESCRIPTION:
 *	Attaches to the mixengined via the SLab APIs, and modifies data stream
 *	by recording the data into a local buffer, and feeding the local buffer
 *	back into the original stream with a time delay.
 *
 * TOC:
 *	main()
 *	echo()
 *	checkParams()
 *
 *******************************************************************/

#include "slabEffects.h"

/*
 * For historic reasons, the descriptor for bus attachments is int *.
 */
int *devPtr;

/*
 * Used by the user interface in the effects rack.
 */
#define EFFECT_NAME "Echo"
#define SHORT_NAME "Echo"

/*
 * These are dependant on the effect - parameters etc.
 */
#define HISTORY_SIZE 131072 /* approx 3 seconds */
int history[HISTORY_SIZE];
int sample = 0, sampleRate;

int delay, decay, feedback;
int Delay = 4096, Decay = 128, Feedback = 128;

int checkParams();
/*
 * When the user interface attaches an effect, it passes two parameters, no
 * switches. This is very rudementary, but will be extended:
 *
 * argv[1] is the controlbuffer segID, argv[2] should be the bus requested
 */
main(argc, argv)
int argc;
char *argv[];
{
	int *busData, count, i;

	if (argc < 3)
	{
		printf("%s: parameter problem\n", argv[0]);
		exit(-1);
	}

	/*
	 * seAttach() will register with the mixengine, and link up the shared
	 * memory control and data structures. It effectively reserves a bus.
	 * The SLAB_EFFECTS_MONO really refers to the first bus attachment. We
	 * can attach to a second one with the SLAB_EFFECTS_STEREO flag. It is
	 * only used as a switch to use a slightly different link up protocol.
	 */
	if ((devPtr = seAttach(atoi(argv[1]), atoi(argv[2]), SLAB_EFFECTS_MONO))
		== (int *) NULL)
	{
		printf("%s: could not attach to requested bus\n", argv[0]);
		exit(-2);
	}

	/*
	 * Clean up the history buffer. Superflous really.
	 */
	bzero(history, HISTORY_SIZE * sizeof(int));

	/*
	 * At this point we can use seAddCtrl() to request up to 8 parameters
	 * which will be shown in the SLab effects user interface. The echo algo
	 * requires delay, decay, and Feedback. The calls return an int which is
	 * used later when requesting paramter values.
	 */
	delay = seAddCtrl(devPtr, "Delay", SLab_Potmeter);
	decay = seAddCtrl(devPtr, "Decay", SLab_Potmeter);
	feedback = seAddCtrl(devPtr, "FeedBack", SLab_Potmeter);

	/*
	 * seInit() returns our fixed buffer size. We always use a fixed buffer as
	 * an internal data representation. No exceptions. Data is manipulated in
	 * this fixed buffer, and then the buffer (pointer) is returned to the bus.
	 */
	if ((count = seInit(devPtr, EFFECT_NAME, SHORT_NAME, checkParams)) < 0)
	{
		seDetach(devPtr);
		exit(-2);
	}

	/*
	 * Init the display
	 */
	sampleRate = seGetOpts(devPtr, SLAB_SAMPLE_RATE);
	checkParams(devPtr, 0);

	/*
	 * This is a forever loop.....
	 */
	while ((busData = seRead(devPtr)) != (int *) -1) {
		if (busData > (int *) 0)
		{
			echo(busData, count);
			seWrite(devPtr, busData);
		}
	}

	/*
	 * seDetach() will deregister with the mixengine, and remove links to the
	 * shared memory control and data structures.
	 */
	if (seDetach(devPtr) < 0)
	{
		printf("%s: could not detach from requested bus\n", argv[0]);
		exit(-3);
	}

	printf("Exiting\n");

	exit(0);
}

echo(busData, count)
register int *busData, count;
{
	register int holder;

#ifdef DEBUG
	printf("echo(%x)\n", busData);
#endif

	for (; count > 0; count--)
	{
		holder = *busData;
		*(busData++) += (history[sample] * Decay) >> 8;
		history[sample] = holder + ((history[sample] * Feedback) >> 8);

		if (++sample >= Delay)
			sample = 0;
	}
}

checkParams(devPtr, param)
int *devPtr, param;
{
	char text[32];

	if (param == delay) {
		Delay = (seGetCtrlVal(devPtr, delay) << 9);
		if (Delay > sampleRate) {
			sprintf(text, "Delay: %2.2f s", (float) Delay/sampleRate);
			seSetCtrlDisp(devPtr, delay, text);
		} else {
			sprintf(text, "Delay: %1.3f ms", (float) Delay/sampleRate);
			seSetCtrlDisp(devPtr, delay, text);
		}
		return;
	}

	if (param == decay) {
		Decay = seGetCtrlVal(devPtr, decay);
		sprintf(text, "Decay: %4i %%", Decay * 100 / 256);
		seSetCtrlDisp(devPtr, decay, text);
		return;
	}

	Feedback = seGetCtrlVal(devPtr, feedback);
	sprintf(text, "Feedback: %4i %%", Feedback * 100 / 256);
	seSetCtrlDisp(devPtr, feedback, text);
}

